import pandas as pd
import glob
import pytz
import datetime
from datetime import date
import os
from os import path
pd.options.mode.chained_assignment = None 
import pathlib
import csv
import sys
import requests
abs_path = pathlib.Path(__file__).parent.absolute()
#print(abs_path)


def get_package_dependencies(
    sha: str, username: str, repo: str, filename: str = "package.json"
) -> list:
    r = requests.get(
        "https://raw.githubusercontent.com/{}/{}/{}/{}".format(
            username, repo, sha, filename
        )
    )
    try:
        result = r.json()

        dependencies = result.get("dependencies")
        if dependencies:
            return dependencies
    except Exception as e:
        return {}
    return None


def get_commits(username: str, repo: str, access_tokens: list = [None], filename: str = "package.json") -> list:
    access_tokens = ["c1efcd24dd9ed43fdf153c57603328bef73c3c3b", "0659f7a9ae2a5451f99bc1536dc9776320e0e149", "0b420fbb5fd82feee1da9cd690d08d88eca9351b", "683d946e3418ec98db60fc31f080cb1194ad27ac", "4ae1d5e61def24fb693296955b1b7072c5c59446", "18397c99faf1f0e636a46a021d2be77e56aebcae", "7bbe429fb29f0b30998122104c99e111a1a6b209", "29d7055440bb9173a7e41081e8eea90d525b78ac", "6f1f7fd8e6b854db0ebea74c3e5825ab8444d594"]
    result = []
    for i in range(1, 10000):
        print("commit page: ", i)
        no_next_page = False
        for x, access_token in enumerate(access_tokens, 1):
            access_failed = False
            print("access attempt: ", x)
            if not access_token:
                url = "https://api.github.com/repos/{}/{}/commits?path={}&page={}".format(
                                    username, repo, filename, i) 
            else:  
                url = "https://api.github.com/repos/{}/{}/commits?path={}&page={}&access_token={}".format(
                                    username, repo, filename, i, access_token
                                )
            r = requests.get(url)
            if r.status_code == 403:
                if not access_token: 
                    print("ERROR: Github PUBLIC API REQUEST LIMIT EXHAUSTED.\nYou need an github access token to perform this analysis.")
                    sys.exit(1)
                if x == len(access_tokens):
                    print("Access Token Exausted")
                    sys.exit(1)
                continue
            commits = r.json()
            if commits:
                for commit in commits:
                    try:
                        sha = commit.get("sha")
                        url = commit.get("html_url")
                        commit = commit.get("commit")
                        author, date, message = "", "", ""
                        if commit:
                            author = commit.get("author").get("name")
                            date = commit.get("committer").get("date")
                            message = commit.get("message")
                        else:
                            print(sha, url, "commit not found")
                        if sha:
                            dependencies = get_package_dependencies(sha, username, repo)
                            if dependencies is None:
                                return result
                            if dependencies:
                                for package_name, version in dependencies.items():
                                    result.append(
                                        {
                                            "sha": sha,
                                            "url": url,
                                            "author": author,
                                            "date": date,
                                            "message": message,
                                            "package_name": package_name,
                                            "version": version,
                                        }
                                    )

                    except Exception as e:
                        access_failed = True
                        print(e)
                        break
            else:
                no_next_page = True
                break
            if not access_failed:
                break

        if no_next_page:
            break

    return result



def step1():
    urls = []
    with open('./repos_list.csv', 'r') as f:
        reader = csv.DictReader(f)
        for row in reader:
            urls.append(row['repo_name'])
        print(urls)

    for url in urls:
        repo, username = url.split("/")[::-1][:2:]
        #print(username, repo)
        output_filePath = os.path.join(abs_path, "OUTPUT", repo + "_dependency_data.csv" )
        if (path.exists(output_filePath)):
            continue
        
        result = get_commits(username,repo)
        
        df = pd.DataFrame()
        if result:
            df = df.append(result, ignore_index=True)
            #df.to_csv("./OUTPUT/"+repo+"_data1.csv", index=False)
            df = df.drop_duplicates()
            df = df[~df["version"].str.contains("/")]
            # fill_dates
            #print("processing:",repo)
            #df = pd.read_csv(input_file)

            df['date'] = pd.to_datetime(df['date'])
            #today = date.today()
            #x = datetime.datetime(2017, 1, 17)
            #print(x)
            dates = pd.date_range(df['date'].min().strftime('%Y-%m-%d'), df['date'].max().strftime('%Y-%m-%d'))
            missing_dates = dates[~dates.isin(df['date'].dt.date)].to_list()    
            avail_dates = dates[dates.isin(df['date'].dt.date)].to_list()[::-1]
            dates = dates.to_list()
            nf = pd.DataFrame()
            for missing_date in missing_dates:
                for last_matched_date in avail_dates:
                    if last_matched_date > missing_date: continue
                    else: break
                rows = df[df['date'].dt.date == last_matched_date].copy(deep=False)
                rows['date'] = rows['date'].apply(lambda x: pytz.utc.localize(missing_date))
                #print(rows)  print to check the progress
                nf = pd.concat([nf, rows])
            df = pd.concat([df, nf])
            
            df['date'] = df.date.astype(str)
            #newdf = df[df['date'].str.contains('2016-')]
            df.to_csv(output_filePath, index=False)
        else:
            print("No Result Found for", username, repo)

    # for input_file in glob.glob(path.join(abs_path, "INPUT", "*.csv")):
    #     output_file  = input_file.replace("INPUT", "OUTPUT")
    #     if (path.exists(output_file)):
    #         continue
    #     print("processing:",input_file)
    #     df = pd.read_csv(input_file)

    #     df['date'] =    pd.to_datetime(df['date'])
    #     #today = date.today()
    #     #x = datetime.datetime(2017, 1, 17)
    #     #print(x)
    #     dates = pd.date_range(df['date'].min().strftime('%Y-%m-%d'), df['date'].max().strftime('%Y-%m-%d'))
    #     missing_dates = dates[~dates.isin(df['date'].dt.date)].to_list()    
    #     avail_dates = dates[dates.isin(df['date'].dt.date)].to_list()[::-1]
    #     dates = dates.to_list()
    #     nf = pd.DataFrame()
    #     for missing_date in missing_dates:
    #         for last_matched_date in avail_dates:
    #             if last_matched_date > missing_date: continue
    #             else: break
    #         rows = df[df['date'].dt.date == last_matched_date].copy(deep=False)
    #         rows['date'] = rows['date'].apply(lambda x: pytz.utc.localize(missing_date))
    #         #print(rows)  print to check the progress
    #         nf = pd.concat([nf, rows])
    #     df = pd.concat([df, nf])
        
    #     df['date'] = df.date.astype(str)
    #     #newdf = df[df['date'].str.contains('2016-')]
    #     df.to_csv(output_file, index=False)

if __name__ == "__main__":
    #import cProfile
    #cProfile.run('identifying_vulnerability_levels()')
    step1()
    pass

